@@ -1,10 +1,10 @@
-/* $Id: global.cc,v 1.81 2003/11/14 01:59:42 bergo Exp $ */
+/* $Id: global.cc,v 1.93 2007/01/20 23:28:57 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -189,6 +189,21 @@ Global::Global() {
   IcsAllObPlayed   = 0;
   IcsAllObObserved = 0;
 
+  SeekTableSortCol = 5;
+  SeekTableSortDir = 0;
+
+  JoystickFD = -1;
+
+  JSCursorAxis = 0;
+  JSBrowseAxis = 2;
+  JSMoveButton = 0;
+  JSNextTabButton = 5;
+  JSPrevTabButton = 4;  
+  JSMode  = 1;
+  JSSpeed = 4;
+
+  joycapture = NULL;
+
   ShowQuickbar=1;
   LowTimeWarningLimit=5;
   RetrieveChannelNames=1;
@@ -268,6 +283,15 @@ Global::Global() {
   RCKeys.push_back("IcsAllObPlayed");  // 44
   RCKeys.push_back("IcsAllObObserved");
   RCKeys.push_back("P2PName");         // 46
+  RCKeys.push_back("SeekTableSortCol");
+  RCKeys.push_back("SeekTableSortDir");// 48
+  RCKeys.push_back("JSCAxis");
+  RCKeys.push_back("JSBAxis"); // 50
+  RCKeys.push_back("JSMButton");
+  RCKeys.push_back("JSNTButton"); // 52
+  RCKeys.push_back("JSPTButton");
+  RCKeys.push_back("JSMode"); // 54
+  RCKeys.push_back("JSSpeed");
 
   PopupHelp = false;
 }
@@ -321,6 +345,15 @@ void Global::removeBoard(Board *b) {
   //  cerr << "<Global::removeBoard> ** board not found\n";
 }
 
+bool Global::effectiveLegalityChecking() {
+  if (CheckLegality) return true;
+
+  if (protocol != NULL)
+    return(protocol->requiresLegalityChecking());
+
+  return false;
+}
+
 void Global::statOS() {
   FILE *p;
   p=popen("uname -s","r");
@@ -343,8 +376,11 @@ void Global::ensureDirectories() {
   if (env.Home.empty()) {
     cerr << _("[eboard] ** no $HOME") << endl;
     return;
+  } else if (strlen(env.Home.c_str()) > 230) {
+    cerr << _("[eboard] ** $HOME is too long") << endl;
+    return;
   }
-  sprintf(z,"%s/.eboard",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard",env.Home.c_str());
 
   tdir=opendir(z);
   if (tdir==NULL)
@@ -353,11 +389,11 @@ void Global::ensureDirectories() {
     closedir(tdir);
     
   if (createDir(z)) return;
-  sprintf(z,"%s/.eboard/craftylog",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard/craftylog",env.Home.c_str());
   createDir(z);
-  sprintf(z,"%s/.eboard/eng-out",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard/eng-out",env.Home.c_str());
   createDir(z);
-  sprintf(z,"%s/.eboard/scripts",env.Home.c_str());
+  snprintf(z,256,"%s/.eboard/scripts",env.Home.c_str());
   createDir(z);
 }
 
@@ -415,7 +451,7 @@ void Global::readRC() {
     if (!p) continue;
     if (p->at(0)=='#') continue;
 
-    for(j=0;j<=46;j++) {
+    for(j=0;j<=55;j++) {
       if (! p->compare(RCKeys[j]) ) {
 	switch(j) {
 	case  0: HilightLastMove =t.tokenvalue(sep); break;
@@ -487,9 +523,18 @@ void Global::readRC() {
 	case 45: IcsAllObObserved    =t.tokenvalue(sep); break;
 	case 46: p=t.token(sep2); memset(P2PName,0,64);
                  p->copy(P2PName,63); break;
+	case 47: SeekTableSortCol = t.tokenvalue(sep); break;
+	case 48: SeekTableSortDir = t.tokenvalue(sep); break;
+	case 49: JSCursorAxis = t.tokenvalue(sep); break;
+	case 50: JSBrowseAxis = t.tokenvalue(sep); break;
+	case 51: JSMoveButton = t.tokenvalue(sep); break;
+	case 52: JSNextTabButton = t.tokenvalue(sep); break;
+	case 53: JSPrevTabButton = t.tokenvalue(sep); break;
+	case 54: JSMode  = t.tokenvalue(sep); break;
+	case 55: JSSpeed = t.tokenvalue(sep); break;
 	} // switch
       } // compare
-    } // for j 0..41
+    } // for j 0..55
   } // while getline
 
   rc.close();
@@ -582,6 +627,17 @@ void Global::writeRC() {
   rc << RCKeys[45] << div << IcsAllObObserved << endl;
   rc << RCKeys[46] << div << P2PName << endl;
 
+  rc << RCKeys[47] << div << SeekTableSortCol << endl;
+  rc << RCKeys[48] << div << SeekTableSortDir << endl;
+
+  rc << RCKeys[49] << div << JSCursorAxis << endl;
+  rc << RCKeys[50] << div << JSBrowseAxis << endl;
+  rc << RCKeys[51] << div << JSMoveButton << endl;
+  rc << RCKeys[52] << div << JSNextTabButton << endl;
+  rc << RCKeys[53] << div << JSPrevTabButton << endl;
+  rc << RCKeys[54] << div << JSMode << endl;
+  rc << RCKeys[55] << div << JSSpeed << endl;
+
   rc.close();
 }
 
@@ -590,7 +646,7 @@ ChessGame * Global::getGame(int num) {
   for(gi=GameList.begin();gi!=GameList.end();gi++)
     if ( (*(*gi)) == num )
       return(*gi);
-  return 0;
+  return NULL;
 }
 
 int Global::nextFreeGameId(int base) {
@@ -730,7 +786,7 @@ void Global::debug(char *klass,char *method,char *data) {
   if (env.Home.empty())
     return;
 
-  sprintf(z,"%s/DEBUG.eboard",env.Home.c_str());
+  snprintf(z,256,"%s/DEBUG.eboard",env.Home.c_str());
 
   ofstream f(z,ios::app);
   if (!f) return;
@@ -761,7 +817,7 @@ void Global::LogAppend(char *msg) {
     msg=_("(message obfuscated -- password mode ?)");
 
   if (CommLog) {
-    sprintf(z,"%s/LOG.eboard",env.Home.c_str());
+    snprintf(z,256,"%s/LOG.eboard",env.Home.c_str());
 
     ofstream f(z,ios::app);
     if (!f) return;
@@ -872,39 +928,6 @@ void * Global::safeMalloc(int nbytes) {
   return(malloc(nbytes + (nbytes % 4)));
 }
 
-GdkFont * Global::loadFont(EboardFont i) {
-  GdkFont *f;
-
-  switch(i) {
-
-  case EF_PlayerFont: // player font (default 18 pt)
-    f=gdk_font_load(PlayerFont);
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--17-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--14-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;       
-
-  case EF_ClockFont: // clock font (default 34 pt)
-    f=gdk_font_load(ClockFont);
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--34-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--25-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--24-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;
-
-  case EF_InfoFont: // info font (default 12 pt)
-    f=gdk_font_load(InfoFont);
-    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--11-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--10-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;
-  default:
-    f=NULL;
-  }
-  return f;
-
-}
-
 void Global::setPieceSet(string &filename,bool chgPieces,bool chgSquares) {
   string oldp,olds;
   PieceSet *oldset=0;
@@ -1023,7 +1046,7 @@ void ChannelSplitter::getChannels(char *ipaddr) {
   if (! global.RetrieveChannelNames)
     return;
 
-  sprintf(destname,"/tmp/eboard-chlist-%s-%d.tmp", ipaddr, getuid() );
+  snprintf(destname,512,"/tmp/eboard-chlist-%s-%d.tmp", ipaddr, getuid() );
   chlist=destname;
 
   if (stat(destname, &age)==0) {
@@ -1034,7 +1057,7 @@ void ChannelSplitter::getChannels(char *ipaddr) {
       goto cs_gc_use_current;
   }
 
-  sprintf(url,"http://eboard.sourceforge.net/ics/%s.txt",ipaddr);
+  snprintf(url,512,"http://eboard.sourceforge.net/ics/%s.txt",ipaddr);
 
   kid=fork();
   if (kid==0) {
@@ -1089,11 +1112,11 @@ const char * ChannelSplitter::getChannelTitle(int n) {
   j=channels.size();
   for(i=0;i<j;i++) {
     if (n==channels[i].number) {
-      sprintf(z,"#%d: %s",n,channels[i].name.c_str());
+      snprintf(z,128,"#%d: %s",n,channels[i].name.c_str());
       return z;
     }
   }
-  sprintf(z,"#%d",n);
+  snprintf(z,128,"#%d",n);
   return z;
 }
 
@@ -1114,7 +1137,7 @@ void ChannelSplitter::createPane(int ch) {
   if (!nb) return;  
   op=new Text();
   
-  sprintf(z,"%s",getChannelTitle(ch) );
+  snprintf(z,64,"%s",getChannelTitle(ch) );
 
   op->show();
   nb->addPage(op->widget,z,-200-ch,true);
@@ -1181,6 +1204,13 @@ void ChannelSplitter::updateChannelScrollBacks() {
     panes[i]->updateScrollBack();
 }
 
+void ChannelSplitter::updateFont() {
+  int i,j;
+  j=panes.size();
+  for(i=0;i<j;i++)
+    panes[i]->updateFont();
+}
+
 TerminalColor::TerminalColor() {
   TextDefault   = 0xeeeeee;
   TextBright    = 0xffffff;
@@ -1221,6 +1251,10 @@ WindowGeometry::WindowGeometry() {
   setNull();
 }
 
+void WindowGeometry::print() {
+  cout << "X,Y,W,H = " << X << "," << Y << "," << W << "," << H << endl;
+}
+
 void WindowGeometry::retrieve(GtkWidget *w) {
   gint a[7];  
   gdk_window_get_geometry(w->window,a,a+1,a+2,a+3,a+4);
@@ -1249,6 +1283,10 @@ void WindowGeometry::read(tstring &t) {
 
 // --------
 
+Desktop::Desktop() {
+  clear();
+}
+
 void Desktop::clear() {
   vector<WindowGeometry *>::iterator i;
   vector<string *>::iterator j;
@@ -1266,7 +1304,7 @@ void Desktop::clear() {
 
   consoles.clear();
   cfilters.clear();
-
+  PanePosition = 0;
 }
 
 void Desktop::read(tstring &t) {
@@ -1275,7 +1313,7 @@ void Desktop::read(tstring &t) {
   wGames.read(t);
   wLocal.read(t);
   wAds.read(t);
-  global.Desk.PanePosition = t.tokenvalue(sep); /* BUG */
+  global.Desk.PanePosition = t.tokenvalue(sep);
 }
 
 void Desktop::writeConsoles(ostream &s, const char *key) {
@@ -1322,7 +1360,7 @@ void Desktop::spawnConsoles(TextSet *ts) {
     dc->show();
     dc->restorePosition(consoles[i]);
     if (cfilters[i]->size()) {
-      strcpy(tmp,cfilters[i]->c_str());
+      g_strlcpy(tmp,cfilters[i]->c_str(),512);
       dc->setFilter(tmp);
     }
   }
