@@ -1,10 +1,10 @@
-/* $Id: position.cc,v 1.54 2003/07/06 18:17:36 bergo Exp $ */
+/* $Id: position.cc,v 1.60 2007/01/20 15:58:43 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -296,7 +296,7 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr) {
        ) 
     {
       if (m[0]=='o') { // cope with o-o for castling (gnu chess 4)
-	strcpy(xlate,m);
+	g_strlcpy(xlate,m,12);
 	for(i=0;xlate[i];i++)
 	  xlate[i]=toupper(xlate[i]);
 	moveStdNotation(xlate,color,Vr);
@@ -433,7 +433,7 @@ void Position::moveStdNotation(char *m,piece color,variant Vr) {
   }
 
   // check whether kings and rooks moved
-  if ((Vr!=WILD)&&(Vr!=SUICIDE))
+  if (IS_NOT_WILD(Vr) && (Vr!=SUICIDE))
     checkCastlingPossibility();
 
   // handle explosions in atomic
@@ -631,7 +631,7 @@ void Position::moveCartesian(int x1,int y1,int x2,int y2,
       }      
   }
 
-  if ((Vr!=WILD)&&(Vr!=SUICIDE))
+  if (IS_NOT_WILD(Vr) && (Vr!=SUICIDE))
     checkCastlingPossibility();
 
   // handle explosions in atomic
@@ -920,6 +920,13 @@ void Position::print() {
     cerr << "\n+---+---+---+---+---+---+---+---+\n";
   }
   cerr << endl;
+
+  cerr << "stock = ";
+  for(i=0;i<2;i++)
+    for(j=0;j<5;j++)
+      cerr << House[j][i] << " ";
+  cerr << endl;
+
 }
 
 /* find the diff from current position to np */
@@ -1077,7 +1084,7 @@ string & Position::getMaterialString(variant Vr) {
 	b+=v;
     }
 
-  sprintf(tmp,_("Material: %d - %d"),w,b);
+  snprintf(tmp,64,_("Material: %d - %d"),w,b);
   MaterialString=tmp;
   return(MaterialString);
 }
@@ -1176,8 +1183,9 @@ bool Position::isMoveLegalCartesian(int x0,int y0,int x1,int y1,piece mycolor,va
     if (dx>1) {
       if ((mycolor==WHITE)&&((y0!=y1)||(y0!=0))) return false;
       if ((mycolor==BLACK)&&((y0!=y1)||(y0!=7))) return false;
-      if (vari==WILD) return true; // wild castling accepted always
+      if (vari==WILD || vari==WILDFR || vari==WILDCASTLE) return true; // wild castling accepted always
       if (vari==SUICIDE) return false; // suicide can't castle
+      if (vari==WILDNOCASTLE) return false; // nocastle shuffle chess
       if (dx!=2) return false;
       if ((x0==4)&&(x1==6)&&(maycastle[mycolor==WHITE?0:1]))
 	if ((square[5][y0]==EMPTY)&&(square[6][y0]==EMPTY))
@@ -1413,7 +1421,7 @@ string & Position::getFEN() {
   if (!ec) fen+="-";
   fen+=" ";
   
-  if (ep[0]>=0) { fen+='a'+ep[0]; fen+='0'+ep[1]; }
+  if (ep[0]>=0) { fen+='a'+ep[0]; fen+='1'+ep[1]; }
   else { fen+='-'; }
   fen+=" ";
   
@@ -1494,6 +1502,6 @@ void Position::setFEN(const char *fen) {
   ep[0]=-1;
   if ((*p)!='-') {
     ep[0]=(*p++)-'a';
-    ep[1]=(*p)-'0';
+    ep[1]=(*p)-'1';
   }  
 }
