@@ -1,10 +1,10 @@
-/* $Id: proto_fics.cc,v 1.101 2003/11/22 04:55:29 bergo Exp $ */
+/* $Id: proto_fics.cc,v 1.119 2007/01/20 15:58:43 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -102,11 +102,15 @@ FicsProtocol::FicsProtocol() {
 
   BeginObserve.set("You are now observing game %n.");
   EndObserve.set("*Removing game %n from observation list*");
+  ObsInfoLine.set("Game %n: %S (*) %S (*) %s %r *");
 
   EndExamine.set("You are no longer examining game %n*");
 
   Style12.set("<12> *");
   Style12House.set("<b1> game %n white [*] black [*]*");
+  /* As of 2007 FICS is sending this before moves even in crazyhouse games */
+  Style12BadHouse.set("<b1> game %n white [*] black [*]%b<-*");
+
   WhiteWon.set    ("{Game %n (%S vs. %S) *} 1-0*");
   BlackWon.set    ("{Game %n (%S vs. %S) *} 0-1*");
   Drawn.set       ("{Game %n (%S vs. %S) *} 1/2-1/2*");
@@ -162,6 +166,7 @@ FicsProtocol::FicsProtocol() {
 
   SgClear.set("<sc>*");
   SgRemove.set("<sr>*");
+  SgRemove2.set("Ads removed:*");
 
   // <s> 8 w=visar ti=02 rt=2194  t=4 i=0 r=r tp=suicide c=? rr=0-9999 a=t f=f
   //    n0   s0       r0    n1  *0  n2  n3  s1    r1       r2    r3      s2  s3
@@ -214,12 +219,13 @@ FicsProtocol::FicsProtocol() {
 	     &WhiteWon,&BlackWon,&Drawn,&Interrupted,&DrawOffer,
 	     &PrivateTell,&News,&MovesHeader,&MovesHeader2,
 	     &MovesHeader3,&MovesHeader4,&MovesBody,&MovesBody2,
-	     &MovesEnd,0);
+	     &MovesEnd,NULL);
   Binder.add(&Kibitz,&Whisper,&Say,&Shout,&cShout,&It,&ChannelTell,
 	     &Seek,&Notify,&SgClear,&SgAdd,&SgAdd2,&SgRemove,
 	     &PTell,&Say2,&Challenge,&LectureBotXTell,
 	     &LectureBotXTellContinued,&GotPartnerGame,
-	     &AllObNone,&AllObFirstLine,&AllObMidLine,0);
+	     &AllObNone,&AllObFirstLine,&AllObMidLine,
+	     &Style12BadHouse,&ObsInfoLine,&SgRemove2,NULL);
 }
 
 FicsProtocol::~FicsProtocol() {
@@ -283,8 +289,7 @@ bool FicsProtocol::parser1(char *T) {
     /* remove prompt, ignore prompt-only lines */
     if (strlen(Prompt.getStarToken(1))>3) {
       memset(pstring,0,1024);
-      strncpy(pstring,Prompt.getStarToken(1),1023);
-      pstring[1023] = 0;
+      g_strlcpy(pstring,Prompt.getStarToken(1),1024);
       T = pstring;
       if (T[0]==' ') T++;
       Binder.prepare(T);
@@ -325,7 +330,7 @@ bool FicsProtocol::parser2(char *T) {
     if (WelcomeMessage.match()) {
       global.input->setPasswordMode(0);
       AuthGone = true;
-      strcpy(nick,WelcomeMessage.getToken(3));
+      g_strlcpy(nick,WelcomeMessage.getToken(3),64);
       UserName.reset();
       UserName.append(new KleeneStarPat());
       UserName.append(new CIExactStringPat(nick));
@@ -360,9 +365,16 @@ bool FicsProtocol::parser3(char *T) {
     return true;
   }
 
+  /* bad (early) zhouse stock lines - ignore with no output unless
+     we are playing bughouse */
+  if (Style12BadHouse.match()) {
+    attachHouseStock(Style12BadHouse,true);
+    return true;
+  }
+
   /* zhouse stock line, parse, end with no output */
   if (Style12House.match()) {
-    attachHouseStock();
+    attachHouseStock(Style12House,false);
     return true;
   }
 
@@ -398,7 +410,17 @@ bool FicsProtocol::parser3(char *T) {
       seekRemove(SgRemove);
       return true;
     }
+  } else {
+
+    if (SgClear.match()) return true;
+    if (SgAdd.match()) return true;
+    if (SgAdd2.match()) return true;
+    if (SgRemove.match()) return true;
+
   }
+  
+  // Ads Removed: msg
+  if (SgRemove2.match()) return true;
 
   return(parser4(T));
 }
@@ -453,11 +475,11 @@ bool FicsProtocol::parser4(char *T) {
   case 1:
     if (MovesHeader.match()) {
       ++RetrievingMoves;
-      strncpy(xxplayer[xxnext], MovesHeader.getSToken(0), 64);
-      strncpy(xxrating[xxnext], MovesHeader.getStarToken(0), 64);
+      g_strlcpy(xxplayer[xxnext], MovesHeader.getSToken(0), 64);
+      g_strlcpy(xxrating[xxnext], MovesHeader.getStarToken(0), 64);
       xxnext=(++xxnext)%2;
-      strncpy(xxplayer[xxnext], MovesHeader.getSToken(1), 64);
-      strncpy(xxrating[xxnext], MovesHeader.getStarToken(1), 64);
+      g_strlcpy(xxplayer[xxnext], MovesHeader.getSToken(1), 64);
+      g_strlcpy(xxrating[xxnext], MovesHeader.getStarToken(1), 64);
       xxnext=(++xxnext)%2;
       return true;
     }
@@ -486,7 +508,12 @@ bool FicsProtocol::parser4(char *T) {
     if (MovesBody2.match()) { retrieve2(MovesBody2); return true; }
     if (MovesEnd.match()) {
       retrievef();
-      RetrievingMoves = 0;
+
+      if (MoveRetrievals.empty())
+	RetrievingMoves = 0;
+      else
+	RetrievingMoves = 1;
+
       return true;
     }
     break;
@@ -502,6 +529,7 @@ bool FicsProtocol::parser4(char *T) {
   -- treat adjournment resume, end with no output
   -- treat observation/examination start/end, end with no output
   -- treat game status retrieval, end with output
+  -- treat game information line (wild/fr, wild/[012] issue)
   -- call next parser
 */
 bool FicsProtocol::parser5(char *T) {
@@ -509,11 +537,11 @@ bool FicsProtocol::parser5(char *T) {
   Binder.prepare(T);
 
   if (CreateGame0.match()) {
-    strncpy(xxplayer[xxnext],CreateGame0.getSToken(0),64);
-    strncpy(xxrating[xxnext],CreateGame0.getStarToken(0),64);
+    g_strlcpy(xxplayer[xxnext],CreateGame0.getSToken(0),64);
+    g_strlcpy(xxrating[xxnext],CreateGame0.getStarToken(0),64);
     xxnext=(++xxnext)%2;
-    strncpy(xxplayer[xxnext],CreateGame0.getSToken(1),64);
-    strncpy(xxrating[xxnext],CreateGame0.getStarToken(1),64);
+    g_strlcpy(xxplayer[xxnext],CreateGame0.getSToken(1),64);
+    g_strlcpy(xxrating[xxnext],CreateGame0.getStarToken(1),64);
     xxnext=(++xxnext)%2;
     return(doOutput(T,-1,false,global.Colors.TextBright));
   }
@@ -536,7 +564,13 @@ bool FicsProtocol::parser5(char *T) {
   }
 
   if (EndObserve.match()) {
-    removeGame(EndObserve);
+    int gn;
+    ChessGame *cg;
+    gn = atoi(EndObserve.getNToken(0));
+    cg = global.getGame(gn);
+    if (cg!=NULL)
+      if (! cg->isOver())
+	removeGame(EndObserve); // if not over, user typed unob himself
     return true;
   }
 
@@ -554,6 +588,25 @@ bool FicsProtocol::parser5(char *T) {
   if (PendingStatus>0 && GameStatus.match()) {
     updateGame(GameStatus);
     --PendingStatus;
+
+  }
+
+  if (ObsInfoLine.match()) {
+    int gm;
+    ChessGame *cg;
+    gm = atoi(ObsInfoLine.getNToken(0));
+    cg = global.getGame(gm);
+    if (cg!=NULL) {
+      char *p;
+      p = ObsInfoLine.getRToken(0);
+      if (!strcmp(p,"wild/fr")) {
+	cg->Variant = WILDFR;
+      } else if (!strcmp(p,"wild/2")) {
+	cg->Variant = WILDNOCASTLE;
+      } else if ((!strcmp(p,"wild/0"))||(!strcmp(p,"wild/1"))) {
+	cg->Variant = WILDCASTLE;
+      }
+    }
     return(doOutput(T,-1,false,global.Colors.TextBright));
   }
 
@@ -611,7 +664,7 @@ bool FicsProtocol::parser7(char *T) {
     case 0:
       if (AllObNone.match()) {
 	if (atoi(AllObNone.getNToken(0)) == AllObReqs.back()) {
-	  sprintf(AllObAcc,"Game %d: no observers.",AllObReqs.back());
+	  snprintf(AllObAcc,1024,"Game %d: no observers.",AllObReqs.back());
 	  AllObReqs.pop_back();
 	  global.status->setText(AllObAcc,20);
 	  return true;
@@ -687,9 +740,9 @@ bool FicsProtocol::parser8(char *T) {
     string x;
     char xn[64], *xp;
     x = "t ";
-    strcpy(xn, tm->getAToken(0));
-    xp = index(xn,'('); if (xp) *xp = 0;
-    xp = index(xn,'['); if (xp) *xp = 0;
+    g_strlcpy(xn, tm->getAToken(0), 64);
+    xp = strchr(xn,'('); if (xp) *xp = 0;
+    xp = strchr(xn,'['); if (xp) *xp = 0;
     x += xn;
     global.ConsoleReply = x;
     global.input->updatePrefix();
@@ -786,8 +839,8 @@ void FicsProtocol::createExaminedGame(int gameid,char *whitep,char *blackp) {
   cg->GameNumber=gameid;
   cg->StopClock=1;
   cg->source = GS_ICS;
-  strncpy(cg->PlayerName[0],whitep,64);
-  strncpy(cg->PlayerName[1],blackp,64);
+  g_strlcpy(cg->PlayerName[0],whitep,64);
+  g_strlcpy(cg->PlayerName[1],blackp,64);
   cg->MyColor=WHITE|BLACK;
   cg->protodata[0]=258;
 
@@ -799,7 +852,7 @@ void FicsProtocol::createExaminedGame(int gameid,char *whitep,char *blackp) {
   b->setCanMove(true);
   b->setSensitive(true);
 
-  sprintf(z,_("Exam.Game #%d"),cg->GameNumber);
+  snprintf(z,64,_("Exam.Game #%d"),cg->GameNumber);
   global.ebook->addPage(b->widget,z,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
 
@@ -810,7 +863,7 @@ void FicsProtocol::createExaminedGame(int gameid,char *whitep,char *blackp) {
 
   cg->acknowledgeInfo();
 
-  sprintf(z,"game %d",cg->GameNumber);
+  snprintf(z,64,"game %d",cg->GameNumber);
   global.network->writeLine(z);
   PendingStatus++;
 }
@@ -819,8 +872,8 @@ void FicsProtocol::sendInitialization() {
   char z[128];
   if (!InitSent) {
     
-    sprintf(z,"set interface eboard %s/%s\niset startpos 1\niset defprompt 1",
-	    global.Version,global.SystemType);
+    snprintf(z,128,"set interface eboard %s/%s\niset startpos 1\niset defprompt 1",
+	     global.Version,global.SystemType);
     global.network->writeLine(z);
     if (global.Premove)
       global.network->writeLine("iset premove 1\n");
@@ -828,6 +881,8 @@ void FicsProtocol::sendInitialization() {
       global.network->writeLine("iset premove 0\n");
     if (global.IcsSeekGraph)
       global.network->writeLine("iset seekinfo 1\niset seekremove 1\nset seek 1");
+    else
+      global.network->writeLine("iset seekremove 1");
     global.network->writeLine("iset lock 1\nstyle 12");
     InitSent = true;
   }
@@ -847,8 +902,8 @@ void FicsProtocol::createScratchPos(int gameid,char *whitep,char *blackp)
   cg->GameNumber=gameid;
   cg->StopClock=1;
   cg->source=GS_ICS;
-  strncpy(cg->PlayerName[0],whitep,64);
-  strncpy(cg->PlayerName[1],blackp,64);
+  g_strlcpy(cg->PlayerName[0],whitep,64);
+  g_strlcpy(cg->PlayerName[1],blackp,64);
   cg->MyColor=WHITE|BLACK;
   cg->protodata[0]=259;
 
@@ -860,7 +915,7 @@ void FicsProtocol::createScratchPos(int gameid,char *whitep,char *blackp)
   b->setCanMove(false);
   b->setSensitive(false);
 
-  sprintf(z,_("Pos: %s vs. %s"),whitep,blackp);
+  snprintf(z,64,_("Pos: %s vs. %s"),whitep,blackp);
   global.ebook->addPage(b->widget,z,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
 
@@ -878,8 +933,8 @@ void FicsProtocol::createGame(ExtPatternMatcher &pm) {
   cg->source=GS_ICS;
   cg->clock_regressive=1;
   cg->GameNumber=atoi(pm.getNToken(0));
-  strncpy(cg->PlayerName[0],pm.getSToken(0),64);
-  strncpy(cg->PlayerName[1],pm.getSToken(1),64);
+  g_strlcpy(cg->PlayerName[0],pm.getSToken(0),64);
+  g_strlcpy(cg->PlayerName[1],pm.getSToken(1),64);
 
   // FICS sends two "Creating" strings when resuming an adjourned
   // game. Weird.
@@ -894,9 +949,9 @@ void FicsProtocol::createGame(ExtPatternMatcher &pm) {
   }
 
   p=knowRating(cg->PlayerName[0]);
-  if (p) strncpy(cg->Rating[0],p,32);
+  if (p) g_strlcpy(cg->Rating[0],p,32);
   p=knowRating(cg->PlayerName[1]);
-  if (p) strncpy(cg->Rating[1],p,32);
+  if (p) g_strlcpy(cg->Rating[1],p,32);
   clearRatingBuffer();
 
   if (!strncmp(cg->PlayerName[0],nick,strlen(cg->PlayerName[0]))) {
@@ -914,18 +969,18 @@ void FicsProtocol::createGame(ExtPatternMatcher &pm) {
     cg->Rated=0;
 
   cg->Variant=REGULAR;
-  if (!strcmp(pm.getRToken(0),"crazyhouse"))
-    cg->Variant=CRAZYHOUSE;
-  if (!strcmp(pm.getRToken(0),"bughouse"))
-    cg->Variant=BUGHOUSE;
-  if (!strcmp(pm.getRToken(0),"suicide"))
-    cg->Variant=SUICIDE;
-  if (!strcmp(pm.getRToken(0),"losers"))
-    cg->Variant=LOSERS;
-  if (!strcmp(pm.getRToken(0),"atomic"))
-    cg->Variant=ATOMIC;
-  if (!strncmp(pm.getRToken(0),"wild",4))
+  if (!strcmp(pm.getRToken(0),"crazyhouse"))    cg->Variant=CRAZYHOUSE;
+  else if (!strcmp(pm.getRToken(0),"bughouse")) cg->Variant=BUGHOUSE;
+  else if (!strcmp(pm.getRToken(0),"suicide"))  cg->Variant=SUICIDE;
+  else if (!strcmp(pm.getRToken(0),"losers"))   cg->Variant=LOSERS;
+  else if (!strcmp(pm.getRToken(0),"atomic"))   cg->Variant=ATOMIC;
+  else if (!strncmp(pm.getRToken(0),"wild",4)) {
     cg->Variant=WILD;
+    if (!strncmp(pm.getRToken(0),"wild/0",6))  cg->Variant=WILDCASTLE;
+    if (!strncmp(pm.getRToken(0),"wild/1",6))  cg->Variant=WILDCASTLE;
+    if (!strncmp(pm.getRToken(0),"wild/2",6))  cg->Variant=WILDNOCASTLE;
+    if (!strncmp(pm.getRToken(0),"wild/fr",7)) cg->Variant=WILDFR;
+  }
 
   global.prependGame(cg);
   global.BoardList.front()->reset();
@@ -953,9 +1008,6 @@ void FicsProtocol::addGame(ExtPatternMatcher &pm) {
 
   cg->clock_regressive=1;
   cg->GameNumber=atoi(pm.getNToken(0));
-
-  cg->STime=999;
-  cg->Inc=999;
   cg->Rated=0;
   cg->Variant=REGULAR;
   cg->source = GS_ICS;
@@ -974,7 +1026,7 @@ void FicsProtocol::addGame(ExtPatternMatcher &pm) {
     cg->protodata[2] = 1;
   }
 
-  sprintf(tab,_("Game #%d"),cg->GameNumber);
+  snprintf(tab,96,_("Game #%d"),cg->GameNumber);
   global.ebook->addPage(b->widget,tab,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
 
@@ -987,7 +1039,7 @@ void FicsProtocol::addGame(ExtPatternMatcher &pm) {
   cg->acknowledgeInfo();
 
   // to gather info about time, variant, player names...
-  sprintf(tab,"game %d",cg->GameNumber);
+  snprintf(tab,96,"game %d",cg->GameNumber);
   global.network->writeLine(tab);
   PendingStatus++;
 }
@@ -1005,7 +1057,7 @@ void FicsProtocol::discardGame(int gameid) {
 
   switch(cg->protodata[0]) {
   case 257: // observed    
-    sprintf(z,"unobserve %d",cg->GameNumber);
+    snprintf(z,64,"unobserve %d",cg->GameNumber);
     break;
   case 258: // examined
     strcpy(z,"unexamine");
@@ -1049,7 +1101,7 @@ void FicsProtocol::innerRemoveGame(int gameid) {
   ChessGame *cg;
 
   cg=global.getGame(gameid);
-  if (!cg)
+  if (cg==NULL)
     return;
 
   // bughouse partner game ?
@@ -1057,7 +1109,7 @@ void FicsProtocol::innerRemoveGame(int gameid) {
     global.bugpane->stopClock();
 
   b=cg->getBoard();
-  if (b==0) return;
+  if (b==NULL) return;
   global.ebook->removePage(gameid);
   global.removeBoard(b);  
   delete(b);
@@ -1116,7 +1168,7 @@ void FicsProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
 }
 
 // attach stock info to bug/zhouse games
-void FicsProtocol::attachHouseStock() {
+void FicsProtocol::attachHouseStock(ExtPatternMatcher &pm, bool OnlyForBughouse) {
   ChessGame *cg;
   int gid;
   char *p;
@@ -1124,25 +1176,30 @@ void FicsProtocol::attachHouseStock() {
 
   global.debug("FicsProtocol","attachHouseStock");
 
+  // Style12House or Style12BadHouse
   //  <b1> game %n white [*] black [*]*
+  //  <b1> game %n white [*] black [*]%b<-*
 
-  gid=atoi(Style12House.getNToken(0));
+  gid=atoi(pm.getNToken(0));
 
   cg=global.getGame(gid);
   if (!cg)
     return;
 
+  if (OnlyForBughouse && cg->Variant != BUGHOUSE)
+    return;
+
   pos=&(cg->getLastPosition());
 
   pos->clearStock();
-  p=Style12House.getStarToken(0);
+  p=pm.getStarToken(0);
 
   //  cerr << "[" << p << "]\n";
 
   for(;*p;p++)
     pos->incStockCount(style12table[*p]);
   
-  p=Style12House.getStarToken(1);
+  p=pm.getStarToken(1);
 
   // cerr << "[" << p << "]\n";
 
@@ -1222,7 +1279,7 @@ void FicsProtocol::parseStyle12(char *data) {
     spawnExamination=false;
     refgame=global.getGame(game);
 
-    if (!refgame)
+    if (refgame==NULL)
       spawnExamination=true;      
     else // v_examine=1 on FICS: played game still on main board but inactive
       if ( (refgame->protodata[0]!=258) && (refgame->isOver()) )
@@ -1236,6 +1293,9 @@ void FicsProtocol::parseStyle12(char *data) {
     game=global.nextFreeGameId(10000);
     createScratchPos(game,plyr[0],plyr[1]);
     break;
+  case -4: // initial position of wild game
+    WildStartPos = pos;
+    return;
   case -2: // observing examined game
   case -1: // I am playing, it's my opponent's turn
   case 1: // I am playing, it's my turn
@@ -1269,7 +1329,7 @@ void FicsProtocol::parseStyle12(char *data) {
     stopclock = 1-stopclock;
 
   refgame=global.getGame(game);
-  if (!refgame) {
+  if (refgame==NULL) {
     cerr << "no such game: " << game << endl;
     return;
   }
@@ -1309,20 +1369,19 @@ void FicsProtocol::parseStyle12(char *data) {
     ackp=1;
   }
 
-  if ( (refgame->STime != itime*60)||(refgame->Inc != inc) ) {
-    refgame->STime=itime*60;
-    refgame->Inc=inc;
+  if ( (refgame->timecontrol.value[0] != itime*60)||(refgame->timecontrol.value[1] != inc) ) {
+    refgame->timecontrol.setIcs(itime*60,inc);
     ackp=1;
   }
 
   if (ackp)
     refgame->acknowledgeInfo();
 
-  strcat(strb," ");
-  strcat(strb,stra);
-  sprintf(strc,"%d.%s%s",
-	  (blacktomove?movenum:movenum-1),
-	  (blacktomove?" ":" ... "),
+  g_strlcat(strb," ",64);
+  g_strlcat(strb,stra,64);
+  snprintf(strc,64,"%d.%s%s",
+	   (blacktomove?movenum:movenum-1),
+	   (blacktomove?" ":" ... "),
 	  strb);
   if (strc[0]=='0')
     strcpy(strc,"0. startpos");
@@ -1394,13 +1453,13 @@ void FicsProtocol::updateGame(ExtPatternMatcher &pm) {
   gm=atoi(pm.getNToken(0));
 
   refgame=global.getGame(gm);
-  if (!refgame) {
+  if (refgame==NULL) {
     PendingStatus++;
     return;
   }
 
-  strncpy(refgame->PlayerName[0],pm.getSToken(0),64);
-  strncpy(refgame->PlayerName[1],pm.getSToken(1),64);
+  g_strlcpy(refgame->PlayerName[0],pm.getSToken(0),64);
+  g_strlcpy(refgame->PlayerName[1],pm.getSToken(1),64);
 
   p=pm.getStarToken(1);
 
@@ -1417,7 +1476,7 @@ void FicsProtocol::updateGame(ExtPatternMatcher &pm) {
     refgame->Variant=SUICIDE;
     break;
   case 'w':
-    refgame->Variant=WILD;
+    if (IS_NOT_WILD(refgame->Variant)) refgame->Variant=WILD;
     break;
   case 'B':
     refgame->Variant=BUGHOUSE;
@@ -1434,23 +1493,23 @@ void FicsProtocol::updateGame(ExtPatternMatcher &pm) {
 
   memset(z,0,8);
   memcpy(z,&p[3],3);
-  refgame->STime=60*atoi(z);
-  refgame->Inc=atoi(&p[7]);
 
+  refgame->timecontrol.setIcs(60*atoi(z),atoi(&p[7]));
   refgame->acknowledgeInfo();
 
-  if ((refgame->Variant!=WILD)&&(refgame->protodata[0]!=258)) {
+  // bughouse partner game, set up names in the bugpane
+  if (refgame->protodata[2]!=0) {
+    global.bugpane->setPlayerNames(refgame->PlayerName[0],
+				   refgame->PlayerName[1]);
+  }
+
+  if (refgame->protodata[0]!=258) {
     MoveRetrievals.push_front(gm);
-    sprintf(z,"moves %d",gm);
+    snprintf(z,32,"moves %d",gm);
     global.network->writeLine(z);
     RetrievingMoves=1;
   }
 
-  // bughouse partner game, set up names in the bugpane
-  if (refgame->protodata[2]) {
-    global.bugpane->setPlayerNames(refgame->PlayerName[0],
-				   refgame->PlayerName[1]);
-  }
 }
 
 void FicsProtocol::gameOver(ExtPatternMatcher &pm, GameResult result) {
@@ -1462,10 +1521,10 @@ void FicsProtocol::gameOver(ExtPatternMatcher &pm, GameResult result) {
   
   game=atoi(pm.getNToken(0));
   reason[63]=0;
-  strncpy(reason,pm.getStarToken(0),64);
+  g_strlcpy(reason,pm.getStarToken(0),64);
 
   refgame=global.getGame(game);
-  if (!refgame) {
+  if (refgame==NULL) {
     cerr << _("no such game: ") << game << endl;
     return;
   }
@@ -1498,14 +1557,14 @@ void FicsProtocol::gameOver(ExtPatternMatcher &pm, GameResult result) {
     {
       if  (refgame->savePGN(global.AppendFile,true)) {
 	char z[128];
-	sprintf(z,_("Game appended to %s"),global.AppendFile);
+	snprintf(z,128,_("Game appended to %s"),global.AppendFile);
 	global.status->setText(z,5);
       }
     }
 
   // refresh seek graph
   if ((global.IcsSeekGraph)&&(refgame->protodata[0]==0))
-	refreshSeeks();
+	refreshSeeks(true);
 
   // remove board ?
   // if game relation is observation and user has the Smart Discard setting on:
@@ -1519,11 +1578,12 @@ void FicsProtocol::gameOver(ExtPatternMatcher &pm, GameResult result) {
   }
 }
 
-void FicsProtocol::refreshSeeks() {
-	if ((global.IcsSeekGraph)&&(global.skgraph)) {
-		global.skgraph->clear();
-		global.network->writeLine("iset seekinfo 1");
-	}
+void FicsProtocol::refreshSeeks(bool create) {
+  if (create) ensureSeekGraph();
+  if ((global.IcsSeekGraph)&&(global.skgraph)) {
+    global.skgraph->clear();
+    global.network->writeLine("iset seekinfo 1");
+  }
 }
 
 int  FicsProtocol::hasAuthenticationPrompts() {
@@ -1556,25 +1616,36 @@ void FicsProtocol::clearMoveBuffer() {
 }
 
 void FicsProtocol::retrieve1(ExtPatternMatcher &pm) {
-  char z[32],mv[16];
+  char z[32],mv[32];
   char tmp[2][32];
   int itmp;
   Position p,q;
+  int gm;
+  ChessGame *cg;
+  variant Vr=REGULAR;
 
   global.debug("FicsProtocol","retrieve1");
 
-  if (!MoveBuf.empty())
-    p=MoveBuf.back();
-  else
-    MoveBuf.push_back(Position());
+  gm=MoveRetrievals.back();
+  cg=global.getGame(gm);
+  if (cg!=NULL) Vr=cg->Variant;
+
+  if (MoveBuf.empty()) {
+    if (IS_WILD(Vr))
+      MoveBuf.push_back(WildStartPos);
+    else
+      MoveBuf.push_back(Position());
+  }
+  p = MoveBuf.back();
+  
 
   itmp=atoi(pm.getNToken(0));
-  strncpy(tmp[0],pm.getRToken(0),32);
-  strncpy(tmp[1],pm.getRToken(1),32);
-  sprintf(z,"%.2d. %s %s",itmp,tmp[0],tmp[1]);
-  strcpy(mv,tmp[0]);
+  g_strlcpy(tmp[0],pm.getRToken(0),32);
+  g_strlcpy(tmp[1],pm.getRToken(1),32);
+  snprintf(z,32,"%.2d. %s %s",itmp,tmp[0],tmp[1]);
+  g_strlcpy(mv,tmp[0],32);
 
-  p.moveStdNotation(mv,WHITE);
+  p.moveStdNotation(mv,WHITE,Vr);
   p.setLastMove(z);
 
   MoveBuf.push_back(p);
@@ -1582,12 +1653,12 @@ void FicsProtocol::retrieve1(ExtPatternMatcher &pm) {
   q=p;
 
   itmp=atoi(pm.getNToken(0));
-  strncpy(tmp[0],pm.getRToken(2),32);
-  strncpy(tmp[1],pm.getRToken(3),32);
-  sprintf(z,"%.2d. ... %s %s",itmp,tmp[0],tmp[1]);
-  strcpy(mv,tmp[0]);
+  g_strlcpy(tmp[0],pm.getRToken(2),32);
+  g_strlcpy(tmp[1],pm.getRToken(3),32);
+  snprintf(z,32,"%.2d. ... %s %s",itmp,tmp[0],tmp[1]);
+  g_strlcpy(mv,tmp[0],32);
 
-  q.moveStdNotation(mv,BLACK);
+  q.moveStdNotation(mv,BLACK,Vr);
   q.setLastMove(z);
 
   MoveBuf.push_back(q);
@@ -1595,22 +1666,33 @@ void FicsProtocol::retrieve1(ExtPatternMatcher &pm) {
 
 void FicsProtocol::retrieve2(ExtPatternMatcher &pm) {
   Position p;
-  char z[32],mv[16];
+  char z[32],mv[32];
   char tmp[2][32];
-  int itmp;
+  int itmp, gm;
+  ChessGame *cg;
+  variant Vr=REGULAR;
 
   global.debug("FicsProtocol","retrieve2");
 
-  if (!MoveBuf.empty())
-    p=MoveBuf.back();
+  gm=MoveRetrievals.back();
+  cg=global.getGame(gm);
+  if (cg!=NULL) Vr=cg->Variant;
+
+  if (MoveBuf.empty()) {
+    if (IS_WILD(Vr))
+      MoveBuf.push_back(WildStartPos);
+    else
+      MoveBuf.push_back(Position());
+  }
+  p = MoveBuf.back();
 
   itmp=atoi(pm.getNToken(0));
-  strncpy(tmp[0],pm.getRToken(0),32);
-  strncpy(tmp[1],pm.getRToken(1),32);
-  sprintf(z,"%.2d. %s %s",itmp,tmp[0],tmp[1]);
-  strcpy(mv,tmp[0]);
+  g_strlcpy(tmp[0],pm.getRToken(0),32);
+  g_strlcpy(tmp[1],pm.getRToken(1),32);
+  snprintf(z,32,"%.2d. %s %s",itmp,tmp[0],tmp[1]);
+  g_strlcpy(mv,tmp[0],32);
 
-  p.moveStdNotation(mv,WHITE);
+  p.moveStdNotation(mv,WHITE,Vr);
   p.setLastMove(z);
 
   MoveBuf.push_back(p);
@@ -1629,21 +1711,22 @@ void FicsProtocol::retrievef() {
   gm=MoveRetrievals.back();
 
   cg=global.getGame(gm);
-  if (!cg) {
+  if (cg==NULL) {
     cerr << _("no such game: ") << gm << endl;
     return;
   }
   MoveRetrievals.pop_back();
 
   p=knowRating(cg->PlayerName[0]);
-  if (p) strncpy(cg->Rating[0],p,32);
+  if (p) g_strlcpy(cg->Rating[0],p,32);
   p=knowRating(cg->PlayerName[1]);
-  if (p) strncpy(cg->Rating[1],p,32);
+  if (p) g_strlcpy(cg->Rating[1],p,32);
   clearRatingBuffer();
 
   cg->updateGame(MoveBuf);
   cg->acknowledgeInfo();
   clearMoveBuffer();
+  WildStartPos.setStartPos();
 }
 
 void FicsProtocol::queryGameList(GameListConsumer *glc) {
@@ -1676,13 +1759,13 @@ void FicsProtocol::sendGameListEntry() {
 
   gn=atoi(GameListEntry.getNToken(0));
   
-  strcpy(frank,GameListEntry.getStarToken(1));
-  strcat(frank,"[");
-  strcat(frank,GameListEntry.getStarToken(2));
-  strcat(frank,"]");
-  strcat(frank,GameListEntry.getStarToken(3));
-  strcat(frank,":");
-  strcat(frank,GameListEntry.getStarToken(4));
+  g_strlcpy(frank,GameListEntry.getStarToken(1),256);
+  g_strlcat(frank,"[",256);
+  g_strlcat(frank,GameListEntry.getStarToken(2),256);
+  g_strlcat(frank,"]",256);
+  g_strlcat(frank,GameListEntry.getStarToken(3),256);
+  g_strlcat(frank,":",256);
+  g_strlcat(frank,GameListEntry.getStarToken(4),256);
 
   lastGLC->appendGame(gn,frank);
 }
@@ -1701,13 +1784,13 @@ void FicsProtocol::sendAdListEntry(char *pat) {
 
 void FicsProtocol::observe(int gameid) {
   char z[64];
-  sprintf(z,"observe %d",gameid);
+  snprintf(z,64,"observe %d",gameid);
   global.network->writeLine(z);
 }
 
 void FicsProtocol::answerAd(int adid) {
   char z[64];
-  sprintf(z,"play %d",adid);
+  snprintf(z,64,"play %d",adid);
   global.network->writeLine(z);
 }
 
@@ -1727,13 +1810,13 @@ void FicsProtocol::clearRatingBuffer() {
 
 void FicsProtocol::exaForward(int n) {
   char z[64];
-  sprintf(z,"forward %d",n);
+  snprintf(z,64,"forward %d",n);
   global.network->writeLine(z);
 }
 
 void FicsProtocol::exaBackward(int n) {
   char z[64];
-  sprintf(z,"backward %d",n);
+  snprintf(z,64,"backward %d",n);
   global.network->writeLine(z);
 }
 
@@ -1745,12 +1828,12 @@ void FicsProtocol::retrieveMoves(ExtPatternMatcher &pm) {
 
   gm=atoi(pm.getNToken(0));  
   refgame=global.getGame(gm);
-  if (!refgame)
+  if (refgame==NULL)
     return;
 
-  if ((refgame->Variant!=WILD)&&(refgame->protodata[0]!=258)) {
+  if (refgame->protodata[0]!=258) {
     MoveRetrievals.push_front(gm);
-    sprintf(z,"moves %d",gm);
+    snprintf(z,64,"moves %d",gm);
     global.network->writeLine(z);
     RetrievingMoves=1;
   }
@@ -1759,7 +1842,7 @@ void FicsProtocol::retrieveMoves(ExtPatternMatcher &pm) {
 // --- seek graph
 
 void FicsProtocol::ensureSeekGraph() {
-  if (!global.skgraph) {
+  if (global.skgraph==NULL) {
     global.skgraph=new SeekGraph();
     global.skgraph->show();
     global.ebook->addPage(global.skgraph->widget,_("Seek Table"),-3);
@@ -1859,17 +1942,17 @@ void FicsProtocol::doPlayerAction(char *player, int id) {
   bool ok=true;
 
   switch(id) {
-  case 0: sprintf(z,"finger %s",player); break;
-  case 1: sprintf(z,"stat %s",player); break;
-  case 2: sprintf(z,"history %s",player); break;
-  case 3: sprintf(z,"ping %s",player); break;
-  case 4: sprintf(z,"log %s",player); break;
-  case 5: sprintf(z,"var %s",player); break;
+  case 0: snprintf(z,256,"finger %s",player); break;
+  case 1: snprintf(z,256,"stat %s",player); break;
+  case 2: snprintf(z,256,"history %s",player); break;
+  case 3: snprintf(z,256,"ping %s",player); break;
+  case 4: snprintf(z,256,"log %s",player); break;
+  case 5: snprintf(z,256,"var %s",player); break;
   default: ok=false;
   }
 
   if (ok) {
-    sprintf(w,_("> [issued from menu] %s"),z);
+    snprintf(w,256,_("> [issued from menu] %s"),z);
     global.output->append(w,global.SelfInputColor);
     global.network->writeLine(z);
   }
@@ -1880,13 +1963,13 @@ void FicsProtocol::doGameAction(int gameid, int id) {
   bool ok=true;
 
   switch(id) {
-  case 0: sprintf(z,"allob %d",gameid); break;
-  case 1: sprintf(z,"ginfo %d",gameid); break;
+  case 0: snprintf(z,256,"allob %d",gameid); break;
+  case 1: snprintf(z,256,"ginfo %d",gameid); break;
   default: ok=false;
   }
 
   if (ok) {
-    sprintf(w,_("> [issued from menu] %s"),z);
+    snprintf(w,256,_("> [issued from menu] %s"),z);
     global.output->append(w,global.SelfInputColor);
     global.network->writeLine(z);
   }
@@ -1895,7 +1978,7 @@ void FicsProtocol::doGameAction(int gameid, int id) {
 void FicsProtocol::prepareBugPane() {
   char z[64];
   PartnerGame = atoi(GotPartnerGame.getNToken(0));
-  sprintf(z,"observe %d",PartnerGame);
+  snprintf(z,64,"observe %d",PartnerGame);
   global.network->writeLine(z);
 }
 
@@ -1903,7 +1986,7 @@ void FicsProtocol::updateVar(ProtocolVar pv) {
   char z[256];
   switch(pv) {
   case PV_premove:
-    sprintf(z,"iset premove %d\n",global.Premove ? 1 : 0);
+    snprintf(z,256,"iset premove %d\n",global.Premove ? 1 : 0);
     global.network->writeLine(z);
     break;
   default:
@@ -1926,7 +2009,7 @@ gboolean fics_allob(gpointer data) {
   proto = (FicsProtocol *) (global.protocol);
   proto->AllObReqs.push_front(cg->GameNumber);
 
-  sprintf(z,"allob %d",cg->GameNumber);
+  snprintf(z,64,"allob %d",cg->GameNumber);
   global.network->writeLine(z);
 
   return TRUE;
@@ -1950,7 +2033,7 @@ bool FicsProtocol::doAllOb1() {
 
   t.set(AllObFirstLine.getStarToken(1));
  
-  sprintf(AllObAcc,"Observing game %d: ",n);
+  snprintf(AllObAcc,1024,"Observing game %d: ",n);
   i=0;
   while((p=t.token(" "))!=0) {
 
@@ -1960,13 +2043,13 @@ bool FicsProtocol::doAllOb1() {
     }
 
     if(i!=0) strcat(AllObAcc,", ");
-    strcat(AllObAcc,p->c_str());
+    g_strlcat(AllObAcc,p->c_str(),1024);
     ++i;
 
     //    cerr << "partial: [" << AllObAcc << "]\n";
 
     if (strlen(AllObAcc) > 128) {
-      strcat(AllObAcc,"...");
+      g_strlcat(AllObAcc,"...",1024);
       break;
     }
   }
@@ -1999,11 +2082,11 @@ void FicsProtocol::doAllOb2() {
     if (strlen(AllObAcc) > 128)
       continue;
 
-    strcat(AllObAcc,", ");
-    strcat(AllObAcc,p->c_str());
+    g_strlcat(AllObAcc,", ",1024);
+    g_strlcat(AllObAcc,p->c_str(),1024);
     ++i;
     if (strlen(AllObAcc) > 128) {
-      strcat(AllObAcc,"...");
+      g_strlcat(AllObAcc,"...",1024);
       break;
     }
   }
