@@ -1,10 +1,10 @@
-/* $Id: widgetproxy.cc,v 1.24 2003/07/06 18:17:36 bergo Exp $ */
+/* $Id: widgetproxy.cc,v 1.36 2007/01/20 15:58:43 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2007 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -91,7 +91,7 @@ GtkWidget * WidgetProxy::PixButton(char **xpmicon, char *caption) {
   t=gtk_label_new(caption);
   gtk_box_pack_start(GTK_BOX(v),t,FALSE,TRUE,0);
 
-  Gtk::show(t,p,v,0);
+  Gtk::show(t,p,v,NULL);
   return b;
 }
 
@@ -113,7 +113,7 @@ void BoxedLabel::setText(char *msg) {
 
 /* titles must be marked with N_(...) */
 ModalDialog::ModalDialog(char *title) {
-  widget=gtk_window_new(GTK_WINDOW_DIALOG);
+  widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
   gtk_window_set_title(GTK_WINDOW(widget),eboard_gettext(title));
 
@@ -225,7 +225,7 @@ UnboundedProgressWindow::UnboundedProgressWindow(const char *_templ) :
   label=gtk_label_new(_("Working..."));
   gtk_widget_set_style(label,style);
   gtk_container_add(GTK_CONTAINER(widget),label);
-  Gtk::show(label,widget,0);
+  Gtk::show(label,widget,NULL);
 
   gtk_signal_connect(GTK_OBJECT(widget),"delete_event",
 		     GTK_SIGNAL_FUNC(upw_delete),0);
@@ -244,7 +244,7 @@ UnboundedProgressWindow::~UnboundedProgressWindow() {
 void UnboundedProgressWindow::setProgress(int v) {
   char z[256];
   int i;
-  sprintf(z,templ,v);
+  snprintf(z,256,templ,v);
   gtk_label_set_text(GTK_LABEL(label),z);
   gtk_widget_queue_resize(label);
   for(i=3;i;i--) // avoid locks with idle hooks
@@ -265,7 +265,7 @@ ColorButton::ColorButton(char *caption, int value) {
   notified=0;
   updateButtonFace();
 
-  Gtk::show(label,widget,0);
+  Gtk::show(label,widget,NULL);
   gtk_signal_connect(GTK_OBJECT(widget),"clicked",
 		     GTK_SIGNAL_FUNC(colorb_click),(gpointer)this);
 }
@@ -366,19 +366,23 @@ TextPreview::TextPreview(GdkWindow *wnd, ColorButton *_bg) {
   bg=_bg;
   bg->hookNotify(this);
 
-  pixmap=gdk_pixmap_new(wnd, 320, 200, -1);
+  pixmap=gdk_pixmap_new(wnd, 400, 300, -1);
 
   widget=gtk_drawing_area_new();
   gtk_widget_set_events(widget,GDK_EXPOSURE_MASK);
-  gtk_drawing_area_size(GTK_DRAWING_AREA(widget),320,200);
+  gtk_drawing_area_size(GTK_DRAWING_AREA(widget),400,300);
   gtk_signal_connect(GTK_OBJECT(widget),"expose_event",
 		     GTK_SIGNAL_FUNC(preview_expose),(gpointer)this);
+
+  pl = gtk_widget_create_pango_layout(widget,NULL);
+  pfd = NULL;
 }
 
 TextPreview::~TextPreview() {
   colors.clear();
   examples.clear();
   gdk_pixmap_unref(pixmap);
+  if (pl!=NULL) g_object_unref(G_OBJECT(pl));
 }
 
 void TextPreview::attach(ColorButton *cb, const char *sample) {
@@ -401,7 +405,7 @@ void TextPreview::thaw() {
 
 void TextPreview::update() {
   GdkGC *gc;
-  GdkFont *font;
+  PangoRectangle prl, pri;
   int i,j,y;
 
   pending=false;
@@ -409,21 +413,30 @@ void TextPreview::update() {
   if (!pixmap) return;
   gc=gdk_gc_new(pixmap);
   gdk_rgb_gc_set_foreground(gc, bg->getColor() );
-  gdk_draw_rectangle(pixmap,gc,TRUE,0,0,320,200);
+  gdk_draw_rectangle(pixmap,gc,TRUE,0,0,400,300);
   
-  font=gdk_font_load(global.ConsoleFont);
-  if (!font) return;
+  if (pfd!=NULL) {
+    pango_font_description_free(pfd);
+    pfd = NULL;
+  }
+  pfd =  pango_font_description_from_string(global.ConsoleFont);
+  if (pfd == NULL) {
+    cerr << "[eboard] TextPreview unable to set font " << global.ConsoleFont << endl;
+    return;
+  }
 
+  pango_layout_set_font_description(pl, pfd);
+
   j=examples.size();
-  y=0;
+  y=2;
   for(i=0;i<j;i++) {
     gdk_rgb_gc_set_foreground(gc, colors[i]->getColor() );
-    y+=gdk_string_height(font, examples[i].c_str() );
-    gdk_draw_string(pixmap,font,gc,5,y,examples[i].c_str() );
-    y+=4;
+    pango_layout_set_text(pl, examples[i].c_str(), -1);
+    pango_layout_get_pixel_extents(pl, &pri, &prl);
+    gdk_draw_layout(pixmap,gc,5,y,pl);
+    y += prl.height + 2;
   }
 
-  gdk_font_unref(font);
   gdk_gc_destroy(gc);
   gtk_widget_queue_resize(widget);
 }
@@ -433,11 +446,11 @@ gboolean preview_expose(GtkWidget *widget,GdkEventExpose *ee,
   TextPreview *me;
   GdkGC *gc;
   me=(TextPreview *)data;
-  if (! me->pixmap) return 0;
-  gc=gdk_gc_new(widget->window);
-  gdk_draw_pixmap(widget->window,gc,me->pixmap,0,0,0,0,320,200);
+  if (me->pixmap == NULL) return false;
+  gc = gdk_gc_new(widget->window);
+  gdk_draw_pixmap(widget->window,gc,me->pixmap,0,0,0,0,400,300);
   gdk_gc_destroy(gc);
-  return 1;
+  return true;
 }
 
 // FileDialog
@@ -477,8 +490,7 @@ bool FileDialog::run() {
   }
 
   if (!destroyed) {
-    strncpy(FileName,gtk_file_selection_get_filename( GTK_FILE_SELECTION(widget) ), 128 );
-    FileName[127]=0;
+    g_strlcpy(FileName,gtk_file_selection_get_filename( GTK_FILE_SELECTION(widget) ), 128 );
     gtk_grab_remove(widget);
     gtk_widget_destroy(widget);
     return true;
@@ -503,23 +515,50 @@ void filedlg_destroy (GtkWidget * w, gpointer data) {
 // ----------- layout box for the clock (previous code was getting out of hand)
 
 LayoutBox::LayoutBox() {
-  GC=0;
-  Target=0;
+  int i;
+  GC=NULL;
+  Target=NULL;
   X=Y=W=H=0;
+  for(i=0;i<5;i++)
+    pfd[i] = NULL;
+  pl = NULL;
 }
 
-void LayoutBox::prepare(GdkPixmap *t, GdkGC *gc, int a, int b, int c, int d) {
+LayoutBox::~LayoutBox() {
+  int i;
+  if (pl!=NULL)
+    g_object_unref(G_OBJECT(pl));
+  for(i=0;i<5;i++)
+    if (pfd[i] != NULL)
+      pango_font_description_free(pfd[i]);
+}
+
+void LayoutBox::prepare(GtkWidget *w, GdkPixmap *t, GdkGC *gc, int a, int b, int c, int d) {
   int i;
   Target = t;
   GC = gc;
   X=a; Y=b, W=c; H=d;
 
   for(i=0;i<5;i++)
-    fontv[i]=0;
+    pfd[i] = NULL;
+
+  pl = gtk_widget_create_pango_layout(w,NULL);
 }
 
-void LayoutBox::setFont(int i,GdkFont *f) {
-  fontv[i%5] = f;
+bool LayoutBox::prepared() {
+  return(GC!=NULL && Target!=NULL && pl!=NULL && pfd[0]!=NULL);
+}
+
+void LayoutBox::setFont(int i,const char *fontname) {
+  i%=5;
+  if (pfd[i] != NULL) {
+    pango_font_description_free(pfd[i]);
+    pfd[i] = NULL;
+  }
+
+  pfd[i] = pango_font_description_from_string(fontname);
+  if (pfd[i] == NULL)
+    cerr << "[eboard] unable to set font " << fontname << endl;
 }
 
 void LayoutBox::setColor(int c) {
@@ -541,28 +580,70 @@ void LayoutBox::drawEllipse(int x,int y,int w,int h,bool fill) {
 void LayoutBox::consumeTop(int v) { Y+=v; H-=v; }
 void LayoutBox::consumeBottom(int v) { H-=v; }
 
-void LayoutBox::drawString(int x,int y, int f, char *s) {
-  gdk_draw_string(Target,fontv[f],GC,x,y,s);
+void LayoutBox::drawString(int x,int y, int f, const char *s) {
+  if (pl != NULL) {
+    pango_layout_set_font_description(pl, pfd[f%5]);
+    pango_layout_set_text(pl,s,-1);
+    gdk_draw_layout(Target,GC,x,y,pl);
+
+    /* 
+    PangoRectangle prl, pri;
+    pango_layout_get_pixel_extents(pl, &pri, &prl);
+
+    gdk_draw_rectangle(Target,GC,FALSE,x+prl.x,y+prl.y,
+		       prl.width,prl.height);
+    gdk_draw_line(Target,GC,x+prl.x,y+prl.y,
+		  x+prl.x+prl.width,y+prl.y+prl.height);
+    gdk_draw_line(Target,GC,x+prl.x,y+prl.y+prl.height,
+		  x+prl.x+prl.width,y+prl.y);
+    */
+
+  }
 }
 
 void LayoutBox::drawString(int x,int y, int f, string &s) {
-  gdk_draw_string(Target,fontv[f],GC,x,y,s.c_str());
+  drawString(x,y,f,s.c_str());
 }
 
-void LayoutBox::drawSubstring(int x,int y, int f, char *s,int len) {
-  gdk_draw_text(Target,fontv[f],GC,x,y,s,len);
+void LayoutBox::drawSubstring(int x,int y, int f, const char *s,int len) {
+  if (pl != NULL) {
+    pango_layout_set_font_description(pl, pfd[f%5]);
+    pango_layout_set_text(pl,s,len);
+    gdk_draw_layout(Target,GC,x,y,pl);
+  }
 }
 
-int LayoutBox::stringWidth(int f,char *s) {
-  return(gdk_string_width(fontv[f],s));
+int LayoutBox::stringWidth(int f,const char *s) {
+  PangoRectangle prl, pri;
+  if (pl != NULL) {
+    pango_layout_set_font_description(pl, pfd[f%5]);
+    pango_layout_set_text(pl,s,-1);
+    pango_layout_get_pixel_extents(pl, &pri, &prl);
+    return(prl.width);
+  } else
+    return 0;
 }
 
-int LayoutBox::substringWidth(int f,char *s,int len) {
-  return(gdk_text_width(fontv[f],s,len));
+int LayoutBox::substringWidth(int f,const char *s,int len) {
+  PangoRectangle prl, pri;
+  if (pl != NULL) {
+    pango_layout_set_font_description(pl, pfd[f%5]);
+    pango_layout_set_text(pl,s,len);
+    pango_layout_get_pixel_extents(pl, &pri, &prl);
+    return(prl.width);
+  } else
+    return 0;
 }
 
 int LayoutBox::fontHeight(int f) {
-  return(gdk_string_height(fontv[f], "Aypb178XW@"));
+  PangoRectangle prl, pri;
+  if (pl != NULL) {
+    pango_layout_set_font_description(pl, pfd[f%5]);
+    pango_layout_set_text(pl,"Aypb178XW@",-1);
+    pango_layout_get_pixel_extents(pl, &pri, &prl);
+    return(prl.height);
+  } else
+    return 0;
 }
 
 bool LayoutBox::drawBoxedString(int x,int y,int w,int h,
@@ -572,7 +653,6 @@ bool LayoutBox::drawBoxedString(int x,int y,int w,int h,
   char *first, *p, *last;
 
   lskip = fontHeight(f) + 2;
-  y += lskip;
 
   // base case: string fits in one line
   if (stringWidth(f,s) <= w) {
@@ -652,7 +732,7 @@ int LayoutBox::drawButton(int x, int y, int w, int h, char *s, int font,
   drawRect(x,y,w,h,false);
 
   if (s)
-    drawString(x+(w-j)/2, y + h - ((3+h-i)/2) - 1,font,s);
+    drawString(x+(w-j)/2, y + ((3+h-i)/2) - 1,font,s);
 
   return w;
 }
@@ -668,7 +748,7 @@ DropBox::DropBox(char *option, ...) {
 
   p = option;
   mi = gtk_menu_item_new_with_label(p);
-  gtk_menu_append(m,mi);
+  gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);
 
   gtk_signal_connect(GTK_OBJECT(mi), "activate",
 		     GTK_SIGNAL_FUNC(dropbox_select),
@@ -683,7 +763,7 @@ DropBox::DropBox(char *option, ...) {
     if (!p) break;
 
     mi = gtk_menu_item_new_with_label(p);
-    gtk_menu_append(m,mi);    
+    gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);    
     gtk_signal_connect(GTK_OBJECT(mi), "activate",
 		       GTK_SIGNAL_FUNC(dropbox_select),
 		       (gpointer) this);
